/* 
   - The Original Code is this file as it was released on September 22, 2003.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2003-2006
   - HJ van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -
*/

const nsIPermissionManager = Components.interfaces.nsIPermissionManager;
const nsICookie = Components.interfaces.nsICookie;

const NO_ROWS_AVAILABLE      = -1;
const NO_ROWS_SELECTED       = 0;
const SINGLE_ROW_SELECTED    = 1;
const MULTIPLE_ROWS_SELECTED = 2;

const BLOCK_BUTTON       = 0;
const ACCEPT_BUTTON      = 1;
const DOMAIN_BUTTON      = 2;
const SESSION_BUTTON     = 3;
const REDIRECT_BUTTON    = 4;
const ASKME_BUTTON       = 5;
const PREF_BUTTON        = 7;
const ADD_BUTTON         = 8;
const ALLOW_SESSION_ONLY = 8; // XXX see bug 225857 comment 7

const BEHAVIOR_BLOCK     = 4;
const BLOCK_COOKIES      = 2;
const BLOCK_POPUP        = true;

const CM_SELECT_ALL      = 0;
const CM_SELECT_NONE     = 1;
const CM_SEPARATOR_1     = 2;
const CM_BLOCK           = 3;
const CM_ACCEPT          = 4;
const CM_ACCEPT_SITE     = 5;
const CM_SESSION_COOKIE  = 6;
const CM_SEPARATOR_2     = 7;
const CM_REDIRECT        = 8;
const CM_ASK_ME          = 9;
const CM_SEPARATOR_3     = 10;
const CM_REMOVE          = 11;

var observerService;
var cookiemanager = null;
var permissionmanager = null;
var popupmanager = null;
var permissionsTree;
var permissions = [];
var deletedPermissions = [];
var gLastPermissionSortColumn = "domain"; // XXX: pending removal!
var gLastPermissionSortAscending = true; // XXX: pending removal!
var entryBox = null;
var gNewHostField = null;

var permissionTypes = [ "cookie", "image", "popup", "object", "script", "frame", "document", "install" ];
var managerTypes = [ "cookieManager", "imageManager", "popupManager", "pluginManager", "scriptManager", "frameManager", "documentManager", "installManager"];

var toolbarButtons = [ "blockButton", "acceptButton", "domainButton", "sessionButton", "redirectButton", "askmeButton", "deleteButton", "prefsButton", "addButton" ];
/***
  * Toolbar button arrays, this should ease the pain of enabling/disabling the buttons
  */
var defaultEnabledButtons    = [ false, false, true,  false, false, false,  true,  true, false ];
var popupEnabledButtons      = [ false, false, false, false, true,  true,   true,  true, false ];
var selectEnabledButtons     = [ false, false, false, false, false, false,  true, false, false ];
var inputEnabledButtons      = [ false, false, false, false, false, false, false, false, true  ];
var cookieEnabledButtons     = [ true,  true,  false,  true, false, false,  true,  true, false ];
var cookieDataEnabledButtons = [ false, false, false, false, false, false,  true,  true, false ];

var prefPanelNames;
var dialogType = "";
var typeID = Number(0);
var permissionsBundle, cookieBundle;
var gLastHost;
var gUpdatingBatch = "";
var gDateService = null;
var gDisplayCookieData = false;
var gConfirmCheckbox = false;
var gCookieDataFromPrefPanel = false;
var gLastCookie = null;
var gUseStackedWindows = false;
var gPriorTypeID = -1;
var gOpenPreferenceWindow = false;

var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPrefBranch);

var gFileName = null;
var cbDatasource = null;
var gBehaviorPrefPrefix = "multizilla.content-blocker.behavior.";
var gPatternPrefPrefix = "multizilla.content-blocker.pattern.for.";
var patternTypeNumbers = [ 1, 3, 4, 5, 6 ]; // skip cookies (0), popups (2) and install (7)
var gWindowScreenY, gWindowScreenX, gWindowHeight, gWindowWidth;
var gActiveCategory = null;
// Global vars used by confirmation dialog
var gEscapeKeyPressed = false;
var gStartupDialogType = null;
var gConfirmForID = 0;

////////////////////////////////////////////////////////////////////////////////
function selectWindowStyling(aWindow)
{
  var documentElement = aWindow.document.documentElement;

  if (readMyPref("multizilla.windows.classic-style", "bool", false))
    documentElement.removeAttribute("extraStyling");
  else
    documentElement.setAttribute("extraStyling", "true");
}
////////////////////////////////////////////////////////////////////////////////
function readMyPref(aPrefString, aPrefType, aDefaultValue)
{
  var browserWindow = getNavigatorWindow(true);
  return browserWindow.mzPreferences.readMyPref(aPrefString, aPrefType, aDefaultValue);
}
////////////////////////////////////////////////////////////////////////////////
function getTabbrowser(aOpenFlag)
{
  var navigatorWindow = getNavigatorWindow(aOpenFlag);

  if (navigatorWindow)
    return navigatorWindow.getBrowser();
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function startup(aDialog)
{
  selectWindowStyling(aDialog);
  observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  gStartupDialogType = window.arguments[0];
  setupIDs(gStartupDialogType, true);
  entryBox = document.getElementById("EntryBox");
  gNewHostField = entryBox.firstChild;
  permissionsTree = document.getElementById("permissionsTree");

  permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"].getService(Components.interfaces.nsIPermissionManager);
  cookiemanager = Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager);
  permissionsBundle = mzGetStringBundle("chrome://multiviews/locale/permission-manager/permissionManager.properties");

  /*** 
    * The following lines are added for backward compatibility i.e. 
    * to make it work with pre/past 2005-03-10 10:36 Mozilla builds
    * see also: https://bugzilla.mozilla.org/show_bug.cgi?id=277097
    */
  try {
    cookieBundle = mzGetStringBundle("chrome://communicator/locale/permissions/cookieViewer.properties");
    var dummy = cookieBundle.GetStringFromName("accepted");
  } catch(ex) {
    cookieBundle = mzGetStringBundle("chrome://communicator/locale/wallet/CookieViewer.properties");
  }
  if (!gDateService) {
    gDateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"].getService(Components.interfaces.nsIScriptableDateFormat);
  }
  prefPanelNames = permissionsBundle.GetStringFromName('prefPanelNames').split(',');
  // Initialize button array
  for (var i = 0; i < toolbarButtons.length; i++) {
    toolbarButtons[i] = document.getElementById(toolbarButtons[i]);
  }
  switchTree(dialogType);
  observerService.addObserver(permissionsReloadDisplay, "perm-changed", false);

  if (window.arguments.length > 1) {
    var prefill = window.arguments[1].replace(/www\./i,'');
    var hostFound = isDuplicate(prefill);

    if (hostFound >= 0) {
      permissionsTree.currentIndex = hostFound;
      permissionsTree.view.selection.toggleSelect(permissionsTree.currentIndex);
    }
    else {
      gNewHostField.value = prefill;
      gLastHost = prefill;
    }
  }
  permissionsTree.focus();
  updateToolbarButtons();
  updateLastHost();
  sizeToContent();
  initWindowOpenType();

  if (gOpenPreferenceWindow || window.arguments && window.arguments[2] == "preferences") {
    gOpenPreferenceWindow = false;
    setTimeout(openPrefOverlay, 0);
  }
}

function setupIDs(managerID, startupFlag)
{
  dialogType = null; // Reset selected manager

  if (managerID == "cookieManagerFromIcon") { // Called from cookie icon on status bar
    observerService.notifyObservers(null, "cookieIcon", "off");
    gDisplayCookieData = true;
    managerID = "cookieManager";
  }
  else if (managerID.indexOf("-Preferences") > 0) { // Called from popup menu on status bar or info message
    gOpenPreferenceWindow = true;
    managerID = managerID.replace(/-Preferences/, '');
  }
  if (startupFlag) {
    if (managerID == "") {
       var prefValue = pref.getComplexValue(prefIdentifier, Components.interfaces.nsISupportsString);
       managerID = prefValue.data.length ? prefValue.data : managerID; 
    }
    else {
      for (typeID in managerTypes) {
        if (managerTypes[typeID] == managerID)
          break;
      }
    }
  }
  else {
    for (typeID in permissionTypes) {
      if (permissionTypes[typeID] == managerID)
        break;
    }
  }
  if (managerID == "mozilla-image-prefs") {
    typeID = 1; // Keep 'Images' active!
  }
  else if (managerID == "mozilla-popup-prefs") {
    typeID = 2; // Keep 'Popups' active!
  }
  typeID = Number(typeID);
  dialogType = permissionTypes[typeID];
  // Select Category Button
  var button = document.getElementById(dialogType);
  button.checked = true;
  button.focus();

  if (!dialogType) {
    alert(permissionsBundle.GetStringFromName('unsupportedManagerSelected'));
    window.close();
  }
}

function shutdown()
{
  observerService.removeObserver(permissionsReloadDisplay, "perm-changed");
  // Save last active Category Button
  var str = Components.classes["@mozilla.org/supports-string;1"]
                      .createInstance(Components.interfaces.nsISupportsString);
  str.data = managerTypes[typeID];
  pref.setComplexValue("multizilla.perm-manager.default-manager", Components.interfaces.nsISupportsString, str);
}

var permissionsReloadDisplay = {

  observe: function(subject, topic, state) {
    // state = changed, added, deleted and cleared
    if (topic == gUpdatingBatch)
      return;
    if (topic == "perm-changed") {
      permissions.length = 0;

      if (gLastPermissionSortColumn == "domain") { // XXX: pending removal!
        gLastPermissionSortAscending = !gLastPermissionSortAscending;
      }
      loadPermissions();
    }
  }
}

var permissionsTreeView = {
  rowCount : 0,
  setTree : function(tree){},
  getImageSrc : function(row,column) {},
  getProgressMode : function(row,column) {},
  getCellValue : function(row,column) {},
  getCellText : function(row,column) {
    var rv = "";

    if (column.id == "domain" || column == "domain") {
      rv = permissions[row].domain;
    } else if (gDisplayCookieData && (column.id == "name" || column == "name")) {
      rv = permissions[row].name;
    } else if (gDisplayCookieData && (column.id == "expires" || column == "expires")) {
      rv = permissions[row].expires;
    } else if (column.id == "capability" || column == "capability") {
      rv = gDisplayCookieData ? getStatusString(permissions[row].capability) : permissions[row].capability;
    }
    return rv;
  },
  getParentIndex : function(row) {return -1;},
  isSeparator : function(index) {return false;},
  isSorted: function() {return false;},
  isContainer : function(index) {return false;},
  selectionChanged: function() {
    updateToolbarButtons();
    updateLastHost();

    if (gDisplayCookieData)
      cookieSelected();
  },
  cycleHeader : function(aColId) {},
  getRowProperties : function(row, prop) {},
  getColumnProperties : function(column, prop) {},
  getCellProperties : function(row, column, prop) {},
  isEditable: function(row, column) { return true; }
 };

function Permission(number, host, domain, type, capability, permission, name, value, isDomain, path, isSecure, expires, policy)
{
  this.number = number;
  this.host = host;
  this.domain = domain;
  this.type = type;
  this.capability = (gDisplayCookieData) ? getStatusString(capability) : capability;
  this.permission = permission;
  // Cookie only data
  if (gDisplayCookieData) {
    this.name = name;
    this.value = value;
    this.isDomain = isDomain;
    this.path = path;
    this.isSecure = isSecure;
    this.expires = getExpiresString(expires);
    this.expiresSortValue = expires;
    this.policy = policy;
  }
}

function loadPermissions()
{
  var count = 0;
  var permText;
  var showPolicyField = false;
  var enumerator = gDisplayCookieData ? cookiemanager.enumerator : permissionmanager.enumerator;
  var useDefault = permissionsBundle.formatStringFromName('pmUseDefault', [prefPanelNames[typeID]], 1);
  var cannotStr = permissionsBundle.formatStringFromName('pmCannotStr', [prefPanelNames[typeID]], 1);
  var canStr = permissionsBundle.formatStringFromName('pmCanStr', [prefPanelNames[typeID]], 1);
  var canSession = permissionsBundle.GetStringFromName('pmCanSession');
  var willRedirect = permissionsBundle.formatStringFromName('pmWillRedirect', [prefPanelNames[typeID]], 1);
  var requireAction = permissionsBundle.formatStringFromName('pmRequireAction', [prefPanelNames[typeID]], 1);

  // Unhide cookie specific columns and splitters when we're displaying cookie data 
  document.getElementById("name").hidden = !gDisplayCookieData;
  document.getElementById("nameSplitter").hidden = !gDisplayCookieData;
  document.getElementById("expires").hidden = true; // !gDisplayCookieData;
  document.getElementById("expiresSplitter").hidden = true; // !gDisplayCookieData;

  while (enumerator.hasMoreElements()) {
    var nextPermission = enumerator.getNext();

    nextPermission = gDisplayCookieData ? nextPermission.QueryInterface(Components.interfaces.nsICookie) : 
                                          nextPermission.QueryInterface(Components.interfaces.nsIPermission);

    var host = nextPermission.host;

    if (gDisplayCookieData) {
      if (nextPermission.policy != nsICookie.POLICY_UNKNOWN)
        showPolicyField = true;
      permissions[count] = new Permission(count++, host, (host.charAt(0)==".") ? host.substring(1, host.length) : host, 
                                          -1, getStatusString(nextPermission.status), null, nextPermission.name, 
                                          nextPermission.value, nextPermission.isDomain, nextPermission.path, 
                                          nextPermission.isSecure, nextPermission.expires, nextPermission.policy);
    }
    else if (nextPermission.type == dialogType) {
      var capability = nextPermission.capability;
      // capability: 0 = Default (based on popup blocker pref) 1 = Allow / 2 = Block / 3 = Redirect to new tab / 4 = Selection Dialog / 8 = Session only cookie
      if (capability == ALLOW_SESSION_ONLY)
        permText = canSession;
      else if (capability < 3)
        permText = (capability == 0) ? useDefault : (capability == 1) ? canStr : cannotStr;
      else 
        permText = (capability == 3) ? willRedirect : requireAction;

      permissions[count] = new Permission(count++, host, (host.charAt(0)==".") ? host.substring(1, host.length) : host, nextPermission.type, permText, capability);
    }
  }
  // show policy field if at least one cookie has a policy
  if (showPolicyField)
    document.getElementById("policyField").removeAttribute("hidden");
}

function loadTree()
{
  permissionsTreeView.rowCount = permissions.length;
  permissionsTree.treeBoxObject.view = permissionsTreeView;

  if (permissions.length == 0 || permissionsTree.currentIndex > (permissions.length-1))
    permissionsTree.currentIndex = -1;
}

function getConfirmation(aWindow, aTitle, aText, aCheckboxText)
{
  gConfirmCheckbox = false;
  var checkbox = {};
  var checkboxText = aCheckboxText ? permissionsBundle.GetStringFromName(aCheckboxText) : null;
  var result = promptService.confirmEx(aWindow, permissionsBundle.GetStringFromName(aTitle), 
                                        permissionsBundle.GetStringFromName(aText),
                                        (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                        (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                        null, null, null, checkboxText, checkbox);

  gConfirmCheckbox = checkbox.value;
  return (result == 0); // Yes returns 0 / No, Esc and X returns 1
}

function deletePermission()
{
  var text = gDisplayCookieData ? 'pmConfirmCookieRemovalText' : 'pmConfirmRemovalText';
  var checkboxText = gDisplayCookieData ? 'pmConfirmCookieRemovalCheckbox' : null;

  if (!getConfirmation(window, 'pmConfirmRemovalTitle', text, checkboxText))
    return;
  var preventNewCookies = gConfirmCheckbox;
  gUpdatingBatch = "perm-changed";
  DeleteSelectedItemFromTree(permissionsTree, permissionsTreeView, permissions, 
                             deletedPermissions, "dummy-button", "dummy-button");

  if (gDisplayCookieData) { // Remove Cookies
    for (var c = 0; c < deletedPermissions.length; c++) {
      cookiemanager.remove(deletedPermissions[c].host, deletedPermissions[c].name, 
                           deletedPermissions[c].path, preventNewCookies);
    }
  }  
  else { // Remove Permissions
    for (var p = 0; p < deletedPermissions.length; p++)
      permissionmanager.remove(deletedPermissions[p].host, deletedPermissions[p].type);
  }
  deletedPermissions.length = 0;
  gUpdatingBatch = "";
}

function handlePermissionKeyPress(aEvent)
{
  if (aEvent.keyCode == 13) { // toggle
    addHostPermission(permissions[permissionsTree.currentIndex].host, -1);
    // permissionsTree.focus();
  }
  else if (aEvent.keyCode == 45 && entryBox.hidden == false) { // insert
    permissionsTree.view.selection.clearSelection();
    gNewHostField.focus();
  }
  else if (aEvent.keyCode == 46) // delete
    deletePermission();
}

function permissionColumnSort(column, updateSelection, startupFlag)
{
  var sortDirection = "natural";
  var sortColumn = document.getElementById(column);

  if (startupFlag) {
    var treeColumn = sortColumn.parentNode.firstChild;

    while (treeColumn) {
      if (treeColumn.localName == "treecol" && treeColumn.hasAttribute("sortActive") && treeColumn.getAttribute("sortActive") == "true") {
        column = treeColumn.id;
        sortColumn = treeColumn;
        sortDirection = sortColumn.getAttribute("sortDirection");
        gLastPermissionSortAscending = (sortDirection != "ascending");
        gLastPermissionSortColumn = column;
      }
      treeColumn = treeColumn.nextSibling;
    }
  }
  gLastPermissionSortAscending = SortTree(permissionsTree, permissionsTreeView, permissions, column, 
                                          gLastPermissionSortColumn, gLastPermissionSortAscending, updateSelection);

  gLastPermissionSortColumn = column; // XXX: pending removal!
  // set attribute 'sortActive' and 'sortDirection' on the index column
  if (!startupFlag) {
    var direction = gLastPermissionSortAscending ? "ascending" : "descending";
    sortColumn.setAttribute("sortActive", "true");
    sortColumn.setAttribute("sortDirection", direction);
  }
  // remove 'sortActive' and 'sortDirection' from the rest of the columns
  var currentCol = sortColumn.parentNode.firstChild;

  while (currentCol) {
    if (currentCol != sortColumn && currentCol.localName == "treecol") {
      currentCol.removeAttribute("sortDirection");
      currentCol.removeAttribute("sortActive");
    }
    currentCol = currentCol.nextSibling;
  }
}

function onHelp(aPageAnchor)
{
  var pageAnchor = "";
  var helpPages = permissionsBundle.GetStringFromName('pmHelpURL');

  if (aPageAnchor) {
    if (typeof aPageAnchor == 'string')
      pageAnchor = "#" + aPageAnchor;
    else
      pageAnchor = "#" + aPageAnchor.target.id.replace(/^permissionManager-/, '');
  }
  var tabbrowser = getTabbrowser(true);
  tabbrowser.selectedTab = tabbrowser.addOrGetTab(helpPages + pageAnchor);
}

// Used from the 'Back' button on permissionManager.xul
function doBackButton()
{
  if (gCookieDataFromPrefPanel)
    openPrefOverlay(); // Cookie Data -> Pref Panel
  else 
    displayCookiePermissions(); // Cookie Data -> Cookie Permissions */
}

function isDuplicate(aHost)
{
  aHost = aHost.replace(/file:/, 'scheme:file');

  for (var i = 0; i < permissions.length; i++) {
    if (permissions[i].host == aHost)
      return i;
  }
  return -1;
}

function addPermission(aPermission)
{
  if (aPermission) {
    addHostPermission(permissions[permissionsTree.currentIndex].host, aPermission);
  }
  else { // Add button and ENTER in textfield
    var prefValue, url = gNewHostField.value;
    var permission = nsIPermissionManager.DENY_ACTION;

    switch(typeID) {
      case 0: prefValue = pref.getIntPref("network.cookie.cookieBehavior");
              permission = (prefValue == BLOCK_COOKIES) ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;
              break;
      case 2: prefValue = pref.getBoolPref("dom.disable_open_during_load");
              permission = (prefValue == BLOCK_POPUP) ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;
              break;
      case 7: permission = nsIPermissionManager.ALLOW_ACTION;
              break;
      default:
              prefValue = pref.getIntPref("multizilla.content-blocker.behavior." + permissionTypes[typeID]);
              permission = (prefValue == BEHAVIOR_BLOCK) ? nsIPermissionManager.ALLOW_ACTION : nsIPermissionManager.DENY_ACTION;
    }
    if (typeID == 0) { // Check 'lifetimePolicy for Cookies
      var lifetimePolicy;

      try { // This pref setting is only available in mozilla build 20040309 and up!
        lifetimePolicy = pref.getIntPref("network.cookie.lifetimePolicy");
      } catch(ex) {
      }
      permission = (lifetimePolicy == 2) ? ALLOW_SESSION_ONLY : permission;
    }
    else if (typeID != 2) { // Skipped for Cookies and Popups
      var uri = Components.classes['@mozilla.org/network/standard-url;1'].createInstance(Components.interfaces.nsIURI);
      uri.spec = url;
      permission = isFiltered(uri, permissionTypes[typeID]) ? nsIPermissionManager.ALLOW_ACTION : permission; // nsIPermissionManager.DENY_ACTION;
    }
    addHostPermission(url, permission);
    var entry = isDuplicate(url);
    permissionsTree.currentIndex = (entry == -1) ? permissionsTree.currentIndex : entry;
    doSelect('none');
    permissionsTree.view.selection.select(permissionsTree.currentIndex);
    permissionsTree.treeBoxObject.ensureRowIsVisible(permissionsTree.currentIndex);
    // Clear entry field and disable Add Button
    gLastHost = "";
    gNewHostField.value = "";
    updateLastHost();
    permissionsTree.focus();
  }
  if (permissionsTree.currentIndex > -1)
    permissionsTree.treeBoxObject.ensureRowIsVisible(permissionsTree.currentIndex)
  updateToolbarButtons();
}

function addHostPermission(aHost, aPermission)
{
  var isFileScheme = aHost.match(/(scheme:file|file:)/i);
  aHost = aHost.replace(/^\s*([-\w]*:\/+)?/, ""); // trim any leading space and scheme
  var URL = isFileScheme ? 'file:' : 'http://' + aHost;
  const nsIIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var URI = nsIIOService.newURI(URL, null, null);

  if (aPermission == -1) { // Toggle permission (true=1/false=0)
    var capability = permissionmanager.testPermission(URI, dialogType);
    aPermission = (capability == nsIPermissionManager.ALLOW_ACTION) ? nsIPermissionManager.DENY_ACTION : nsIPermissionManager.ALLOW_ACTION;
  }
  var firstVisibleRow = permissionsTree.treeBoxObject.getFirstVisibleRow();
  permissionmanager.add(URI, dialogType, aPermission);
  loadTree();

  if (firstVisibleRow)
    permissionsTree.treeBoxObject.scrollToRow(firstVisibleRow);

  if (permissions.length > 0)
    permissionColumnSort("domain", false, true);
  updateToolbarButtons();
}

function displayTreeMessage(dialogType)
{
  var treeMessageBox = document.getElementById('treeStackBox');

  if (dialogType == 'cookie' || dialogType == 'popup' || dialogType == 'install') {
    if (!treeMessageBox.hasAttribute('collapsed'))
      treeMessageBox.setAttribute('collapsed', 'true');
    return;
  }
  else {
    var behavior = 0;

    try {
      behavior = pref.getIntPref(gBehaviorPrefPrefix + dialogType);
    } catch (ex) {
      pref.setIntPref(gBehaviorPrefPrefix + dialogType, 0);
    }
    if (behavior == 0)
      treeMessageBox.removeAttribute('collapsed');
    else if (!treeMessageBox.hasAttribute('collapsed'))
      treeMessageBox.setAttribute('collapsed', 'true');
  }
}

function switchTree(dialogType, skipCategoryTest)
{
  if (!skipCategoryTest && dialogType == gActiveCategory)
    return;

  permissions = [];
  gLastPermissionSortColumn = ""; // XXX: pending removal!
  gLastPermissionSortAscending = false; // XXX: pending removal!

  setupIDs(dialogType, false);
  gActiveCategory = dialogType; // keep the selected category panel
  // Are we displaying cookie data?
  gDisplayCookieData = (dialogType == "cookie" && gDisplayCookieData);
  // dump("\ngDisplayCookieData: " + gDisplayCookieData);
  gLastCookie = null;
  // If yes, hide 'Toggle' and unhide 'Session' button
  toolbarButtons[DOMAIN_BUTTON].hidden = (dialogType == "cookie");
  toolbarButtons[SESSION_BUTTON].hidden = (dialogType != "cookie");
  document.getElementById('backButton').hidden = (!gDisplayCookieData || gStartupDialogType == "cookieManagerFromIcon"); // true;

  if (gStartupDialogType == "cookieManagerFromIcon")
    gStartupDialogType = "";

  // Hide entry field and display cookie data box when needed
  document.getElementById('cookieDataBox').hidden = !gDisplayCookieData;
  entryBox.hidden = (dialogType == "cookie" && gDisplayCookieData);
  document.getElementById('StoredCookiesButton').hidden = (dialogType != "cookie" || gDisplayCookieData);

  loadPermissions();
  loadTree();
  permissionsTree.focus();
  permissionsTree.view.selection.select(-1);
  var hostFoundOnRow = -1;

  if (permissionsTreeView.rowCount > 0) {
    permissionColumnSort("domain", false, true);

    if (gLastHost) {
      hostFoundOnRow = isDuplicate(gLastHost);

      if (hostFoundOnRow >= 0) {
        gNewHostField.value = "";
        toolbarButtons[ADD_BUTTON].disabled = true;
        permissionsTree.currentIndex = hostFoundOnRow;
        permissionsTree.view.selection.toggleSelect(permissionsTree.currentIndex);
        permissionsTree.treeBoxObject.ensureRowIsVisible(permissionsTree.currentIndex);
      }
    }
  }
  if (gLastHost && hostFoundOnRow == -1) {
    gNewHostField.value = gLastHost;
    toolbarButtons[ADD_BUTTON].disabled = gDisplayCookieData;
  }
  displayTreeMessage(dialogType);
  selectDefaultButton();
}

function updateTextField()
{
  if (gNewHostField.value == "") {
    updateToolbarButtons();
    return;
  }
  for (var i = 0; i < inputEnabledButtons.length; i++)
    toolbarButtons[i].disabled = !inputEnabledButtons[i];
}

function updateLastHost()
{
  var currentHost = gNewHostField.value;

  if (permissionsTreeView.rowCount > 0 && permissionsTree.currentIndex > -1) {
    if (permissionsTree.currentIndex <= permissionsTreeView.rowCount) {
      var selectedRows = GetTreeSelections(permissionsTree);

      if (selectedRows.length == 1)
        gLastHost = permissions[permissionsTree.currentIndex].domain || null;
    }
  }
  if (currentHost != "" && currentHost != gLastHost)
    gNewHostField.value = "";
  toolbarButtons[ADD_BUTTON].disabled = (entryBox.hidden || currentHost == "");
}

function updateToolbarButtons(aInputFlag)
{
  var i = 0;
  var buttons = defaultEnabledButtons;
  var selectedRows = GetTreeSelections(permissionsTree);

  if (permissionsTreeView.rowCount < 1 || permissionsTree.currentIndex < 0 || selectedRows.length == 0) {
    for (i = 0; i < toolbarButtons.length; i++)
      toolbarButtons[i].disabled = true;
  }
  else {
    var permission = gDisplayCookieData ? getStatusString(permissions[permissionsTree.currentIndex].value) : permissions[permissionsTree.currentIndex].permission;

    if (selectedRows.length > 1)
      buttons = selectEnabledButtons;
    else if (dialogType == 'popup') {
      var redirectorEnabled = pref.getBoolPref('multizilla.window-open.redirector');
      popupEnabledButtons[REDIRECT_BUTTON] = redirectorEnabled && (permission != 3);
      popupEnabledButtons[ASKME_BUTTON] = redirectorEnabled && (permission != 4);
      buttons = popupEnabledButtons;
    }
    else if (dialogType == 'cookie')
      buttons = gDisplayCookieData ? cookieDataEnabledButtons : cookieEnabledButtons;
    // Disable buttons that shouldn't be enabled at this moment
    for (i = 0; i < buttons.length; i++)
      toolbarButtons[i].disabled = !buttons[i];
    // Enable Allow, Block and Session buttons based on the selected tree row (read permission)
    if (!gDisplayCookieData && selectedRows.length == 1) {
      // var permission = gDisplayCookieData ? getStatusString(permissions[permissionsTree.currentIndex].value) : permissions[permissionsTree.currentIndex].permission;
      toolbarButtons[BLOCK_BUTTON].disabled = (permission == nsIPermissionManager.DENY_ACTION);
      toolbarButtons[ACCEPT_BUTTON].disabled = (permission == nsIPermissionManager.ALLOW_ACTION);
      // Disable button, because the Content Blocker is not ready for this change!
      toolbarButtons[DOMAIN_BUTTON].disabled = true; // (permission == 8);
      toolbarButtons[SESSION_BUTTON].disabled = (permission == 8);
    }
  }
  toolbarButtons[PREF_BUTTON].disabled = false; // The pref button should always be enabled!
}

function mzGetStringBundle(aPath)
{
  var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                   .getService(Components.interfaces.nsIStringBundleService); 
  return strBundleService.createBundle(aPath);
}

function doSelect(aSelection)
{
  switch (aSelection) {
    case "all": // Called from the context menu, or Ctrl+A ?
         permissionsTree.view.selection.selectAll();
         break;
    case "none": // Called from the context menu, or Ctrl+Z ?
         permissionsTree.view.selection.clearSelection();
         permissionsTree.view.selection.select(permissionsTree.currentIndex);
         break;
  }
  permissionsTree.focus();
}

function initContextMenu(aEvent)
{
  var contextMenuItems = aEvent.originalTarget.childNodes;

  if (typeID != gPriorTypeID) {
    if (!gDisplayCookieData) {
      contextMenuItems[CM_BLOCK].label = permissionsBundle.formatStringFromName('pmBlock', [prefPanelNames[typeID]], 1);
      contextMenuItems[CM_ACCEPT].label = permissionsBundle.formatStringFromName('pmAllow', [prefPanelNames[typeID]], 1);
      contextMenuItems[CM_SESSION_COOKIE].label = permissionsBundle.formatStringFromName('pmSession', [prefPanelNames[typeID]], 1);
    }
    var removeLabelText = gDisplayCookieData ? permissionsBundle.GetStringFromName('pmDeleteCookies') : permissionsBundle.GetStringFromName('pmDelete');
    contextMenuItems[CM_REMOVE].label = removeLabelText;
  }
  gPriorTypeID = gDisplayCookieData ? "cookieData" : typeID;

  var i, treeState = getCurrentTreeState();

  if (treeState == NO_ROWS_AVAILABLE) {
    for (i = 0; i < contextMenuItems.length; i++) {
      contextMenuItems[i].setAttribute('disabled', 'true');
    }
    return;
  }
  var treeRows = permissionsTree.treeBoxObject.view.rowCount;

  if (treeState == MULTIPLE_ROWS_SELECTED) {
    contextMenuItems[CM_SELECT_ALL].setAttribute('disabled', 'true');
    contextMenuItems[CM_SELECT_NONE].removeAttribute('disabled');

    if (!gDisplayCookieData) {
      // Disable the Block, Accept and Accept Site menu items
      for (i = CM_SEPARATOR_1; i < CM_SEPARATOR_2; i++) {
        contextMenuItems[i].setAttribute('disabled', 'true');
      }
    }
  }
  else { // NO_ROWS_SELECTED || SINGLE_ROW_SELECTED
    if (treeRows > 1) 
      contextMenuItems[CM_SELECT_ALL].removeAttribute('disabled');
    contextMenuItems[CM_SELECT_NONE].setAttribute('disabled', 'true');

    if (treeState == NO_ROWS_SELECTED) {
      if (!gDisplayCookieData) {
        // Disable the Block, Accept and Accept Site menu items
        for (i = CM_SEPARATOR_1; i < CM_SEPARATOR_2; i++) {
          contextMenuItems[i].setAttribute('disabled', 'true');
        }
      }
      contextMenuItems[CM_REMOVE].setAttribute('disabled', 'true');
    }
    else  {
      if (!gDisplayCookieData) {
        // Enable some of menu items, based on the permission for the currentIndex
        var permission = gDisplayCookieData ? getStatusString(permissions[permissionsTree.currentIndex].value) : permissions[permissionsTree.currentIndex].permission;
        contextMenuItems[CM_BLOCK].setAttribute('disabled', (permission == nsIPermissionManager.DENY_ACTION) ? 'true' : 'false');
        contextMenuItems[CM_ACCEPT].setAttribute('disabled', (permission == nsIPermissionManager.ALLOW_ACTION) ? 'true' : 'false');
        contextMenuItems[CM_ACCEPT_SITE].setAttribute('disabled', (permission == 8) ? 'true' : 'false');
        contextMenuItems[CM_SESSION_COOKIE].setAttribute('disabled', (permission == 8) ? 'true' : 'false');
      }
      contextMenuItems[CM_REMOVE].removeAttribute('disabled');
      // Keep this array in sync with the context menu items and permissions!
      var permissionMenuIndex = [ 0, 0, 0, 2, 1, 0, 8, 0, 3, 4, 0, 0, 0, 0, 0 ];
      // locate menu item that should display the radio dot 
      for (i = 3; i < 10; i ++) {
        if (permissionMenuIndex[i] == 0) // Skip unwanted menu items
          continue;
        else {
          if (permissionMenuIndex[i] == permission) {
            contextMenuItems[i].setAttribute("checked", "true");
          }
          else 
            contextMenuItems[i].removeAttribute("checked");
        }
      }
    }
  }
  if (dialogType == "popup") {
    // Enable MultiZilla's 'New Window Redirector' menu items based on pref setting
    var redirectorEnabled = pref.getBoolPref('multizilla.window-open.redirector');
    contextMenuItems[CM_REDIRECT].setAttribute('disabled', (!redirectorEnabled || permission == 3) ? 'true' : 'false'); // Redirect Popups to new tabs
    contextMenuItems[CM_ASK_ME].setAttribute('disabled', (!redirectorEnabled || permission == 4) ? 'true' : 'false'); // Ask me before opening Popups
  }
  // Some of the menu items are specifically tailored for a given category, or 
  // when cookie data is displayed, so we need to hide the unwanted menu items.
  contextMenuItems[CM_BLOCK].hidden = gDisplayCookieData;
  contextMenuItems[CM_ACCEPT].hidden = gDisplayCookieData;
  contextMenuItems[CM_ACCEPT_SITE].hidden = true; // hidden for now!
  contextMenuItems[CM_SESSION_COOKIE].hidden = (dialogType != "cookie" || gDisplayCookieData);
  contextMenuItems[CM_SEPARATOR_2].hidden = (dialogType != "popup");
  contextMenuItems[CM_REDIRECT].hidden = (dialogType != "popup");
  contextMenuItems[CM_ASK_ME].hidden = (dialogType != "popup");
  contextMenuItems[CM_SEPARATOR_3].hidden = gDisplayCookieData;
}

function getCurrentTreeState()
{
  if (permissionsTree.treeBoxObject.view.rowCount < 1)
    return NO_ROWS_AVAILABLE;

  var selectedRows = GetTreeSelections(permissionsTree).length;

  if (selectedRows > 1)
    return MULTIPLE_ROWS_SELECTED;
  else if (selectedRows == 1)
    return SINGLE_ROW_SELECTED;
  return NO_ROWS_SELECTED;
}

function openPrefOverlay()
{
  // gDisplayCookieData = false;
  // gCookieDataFromPrefPanel = false;
  document.getElementById('backButton').hidden = (!gDisplayCookieData); // true;
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/permission-manager/preferences/permissionManagerPrefWindow.xul", 
                    "", winFeatures, window, "");
}

function getExpiresString(expires)
{
  if (expires) {
    var date = new Date(1000 * expires);
    var expiry = "";

    try {
      expiry = gDateService.FormatDateTime("", gDateService.dateFormatLong, gDateService.timeFormatSeconds, 
                                           date.getFullYear(), date.getMonth()+1, date.getDate(), 
                                           date.getHours(), date.getMinutes(), date.getSeconds());
    }
    catch(ex) {
      // Die silently
    }
    return expiry;
  }
  return cookieBundle.GetStringFromName("AtEndOfSession");
}

function getStatusString(status)
{
  switch (status) {
    case nsICookie.STATUS_ACCEPTED:
      return cookieBundle.GetStringFromName("accepted");
    case nsICookie.STATUS_FLAGGED:
      return cookieBundle.GetStringFromName("flagged");
    case nsICookie.STATUS_DOWNGRADED:
      return cookieBundle.GetStringFromName("downgraded");
  }
  return "";
}

function getPolicyString(policy)
{
  switch (policy) {
    case nsICookie.POLICY_NONE:
      return cookieBundle.GetStringFromName("policyUnstated");
    case nsICookie.POLICY_NO_CONSENT:
      return cookieBundle.GetStringFromName("policyNoConsent");
    case nsICookie.POLICY_IMPLICIT_CONSENT:
      return cookieBundle.GetStringFromName("policyImplicitConsent");
    case nsICookie.POLICY_EXPLICIT_CONSENT:
      return cookieBundle.GetStringFromName("policyExplicitConsent");
    case nsICookie.POLICY_NO_II:
      return cookieBundle.GetStringFromName("policyNoIICollected");
  }
  return "";
}

function cookieSelected()
{
  var selectedRows = GetTreeSelections(permissionsTree);

  if (permissionsTreeView.rowCount < 1 || permissionsTree.currentIndex < 0 || selectedRows.length == 0 || selectedRows.length > 1) {
    if (gLastCookie)
      clearCookieProperties();
    return;
  }
  var selectedRow = selectedRows[0];

  gLastCookie = selectedRow;
  var value, field;

  var props = [
    {id: "ifl_name", value: permissions[selectedRow].name},
    {id: "ifl_value", value: spliceCookieData(permissions[selectedRow].value, 70)}, 
    {id: "ifl_isDomain", value: permissions[selectedRow].isDomain ? cookieBundle.GetStringFromName("domainColon") : cookieBundle.GetStringFromName("hostColon")},
    {id: "ifl_host", value: permissions[selectedRow].host},
    {id: "ifl_path", value: permissions[selectedRow].path},
    {id: "ifl_isSecure", value: permissions[selectedRow].isSecure ? cookieBundle.GetStringFromName("forSecureOnly") : cookieBundle.GetStringFromName("forAnyConnection")},
    {id: "ifl_expires", value: permissions[selectedRow].expires},
    {id: "ifl_policy", value: getPolicyString(permissions[selectedRow].policy)}
  ];

  for (var i = 0; i < props.length; i++)
  {
    field = document.getElementById(props[i].id);
    field.value = props[i].value;
  }
}

function spliceCookieData(str, length) 
{
  var start = 0;
  var result = "";

  while (start < str.length) {
    result += str.slice(start, (start+length)) + " ";
    start += length;
  }
  return result;
}

function clearCookieProperties()
{
  var properties = ["ifl_name", "ifl_value", "ifl_host", "ifl_path", "ifl_isSecure", "ifl_expires", "ifl_policy"];

  for (i in properties)
    document.getElementById(properties[i]).value = "";
}

function displayCookieData()
{
  gDisplayCookieData = true;
  switchTree("cookie", true);

  if (permissionsTree.currentIndex == -1)
    clearCookieProperties();
}

function displayCookiePermissions()
{
  gDisplayCookieData = false;
  gCookieDataFromPrefPanel = false;
  switchTree("cookie", true);
}

function selectDefaultButton()
{
  var defaultButton;
  var backButton = document.getElementById('backButton');
  var acceptButton = document.documentElement.getButton('accept');
  var cancelButton = document.documentElement.getButton('cancel');

  backButton.removeAttribute('default');
  acceptButton.removeAttribute('default');
  cancelButton.removeAttribute('default');

  if (backButton.hidden)
    defaultButton = acceptButton;
  else
    defaultButton = backButton;
  // defaultButton.setAttribute('default', 'true');
}

function selectFile(aWindow, title, mode)
{
  gFileName = null; // reset global filename
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  mode = (mode == 'open') ? nsIFilePicker.modeOpen : nsIFilePicker.modeSave;

  filePicker.init(aWindow, title, mode);
  filePicker.appendFilter("RDF", "*.rdf");
  filePicker.defaultString = "content-blocker.rdf";
  window.filePickerParent = aWindow; // .parent;
  var filePickerState = filePicker.show();

  if (filePickerState != nsIFilePicker.returnCancel) {
    if (filePicker.fileURL.spec && filePicker.fileURL.spec.length > 0)
      gFileName = filePicker.fileURL.spec;
  }
  return filePickerState;
}

function exportFilters(aWindow, title, mode)
{
  var state = selectFile(aWindow, title, mode);

  if (state == 1) // Cancel
    return;
  else if (state == 2) { // Ok, overwrite existing file, ask for confirmation
    if (!getConfirmation(aWindow, 'cbExportManager', 'cbExportConfirm'))
      return;
  }
  if (gFileName.match('.rdf') == null)
    gFileName += ".rdf";
  cbDatasource = mzGetNewOrCurrentDSByFilename(gFileName, true, null, true);
  var saved = 0;
  // Scan through the array for valid type numbers (skips cookies/popups)
  for (var i = 0; i < patternTypeNumbers.length; i++) {
    var index = patternTypeNumbers[i];
    var resourceID = "multizilla:content-blocker:" + permissionTypes[index];
    // dump("\nresourceID: " + resourceID);
    var patterns = getPatternPrefFor(permissionTypes[index]);
    // dump("\npatterns:" + patterns);
    if (patterns.length > 1)
      saved += writePatternsFor(permissionTypes[index], resourceID, patterns, state);
  }
  title = permissionsBundle.GetStringFromName('cbExportManager');
  var text = permissionsBundle.formatStringFromName('pmSavedFilterNotification', [saved], 1);
  promptService.alert(aWindow, title, text);
  cbDatasource = null;
}

function writePatternsFor(type, resourceID, patterns, overWriteFlag)
{
  var index = 0
  var patternCount = patterns.length;
  // alert(type + "\n" + resourceID + "\n" + patternCount + "\n" + overWriteFlag);
  if (overWriteFlag == 2)
    removeEntriesFor(type);
		
  try {
    var entries = mzConstructRDFContainer(cbDatasource, 'seq', resourceID, false);

    for (; index < patternCount; index++) {
      var property = RDF.GetLiteral(patterns[index]);
      entries.InsertElementAt(property, (index + 1), true);
    }
  } catch(ex) {
    // continue
  }
  cbDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  return(index);
}

function getPatternPrefFor(type)
{
  var patterns = new Array();

  try {
    var prefIdentifier = gPatternPrefPrefix + type;
    var prefValue = pref.getComplexValue(prefIdentifier, Components.interfaces.nsISupportsString);
    patterns = prefValue.data.replace(/\s/g, '').split(',');
    patterns = patterns.sort();
  } catch(ex) {
    dump("\nERROR: Initializing patterns for: " + type + " Failed... \nRecovering...");
    var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
    str.data = "";
    pref.setComplexValue(prefIdentifier, Components.interfaces.nsISupportsString, str);
  }
  return patterns;
}

function cbEntries(aResourceID)
{
  var entries = mzRDFCUtils.MakeSeq(cbDatasource, RDF.GetResource(aResourceID));

  if (!entries)
    return null;
  if (!entries.GetElements())
    return null;
  return entries;
}

function removeEntriesFor(aPermissionType)
{
  var resourceIDs = new Array();
  resourceIDs.push("urn:" + aPermissionType);
  resourceIDs.push("multizilla:content-blocker:" + aPermissionType);

  for (var i = 0; i < resourceIDs.length; i++) {
    var resource = RDF.GetResource(resourceIDs[i]);
    // alert(resourceIDs[i]);
    if (mzRDFCUtils.IsEmpty(cbDatasource, resource))
      continue; // Skip empty Seq's
    else if (mzRDFCUtils.IsSeq(cbDatasource, resource)) {
      var index = cbEntries(resourceIDs[i]).GetCount();

      for (index; index > 0; index--)
        cbEntries(resourceIDs[i]).RemoveElementAt(index, true);
    }
  }
  cbDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}

function importFilters(aWindow, title, mode)
{
  var gStatus = selectFile(aWindow, title, mode);

  if (gStatus == 1) // Cancel
    return;
  if (gFileName.match('.rdf') == null)
    gFileName += ".rdf";

  cbDatasource = mzGetNewOrCurrentDSByFilename(gFileName, false, null, true);
  var dsStats = getDataSourceStatistics();

  if (dsStats == null) {
    promptService.alert(aWindow, permissionsBundle.GetStringFromName('cbImportFileErrorTitle'),
                        permissionsBundle.GetStringFromName('cbImportFileErrorText'));
    return;
  }
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  aWindow.openDialog("chrome://multiviews/content/permission-manager/miscellaneous/permissionManagerImportExport.xul", 
                     "", winFeatures, window, dsStats);
}

function getDataSourceStatistics()
{
  var failures = 0;
  var dsStatistics = new Array();

  for (var i = 0; i < patternTypeNumbers.length; i++) {
    var index = patternTypeNumbers[i];
    var resourceID = "multizilla:content-blocker:" + permissionTypes[index];
    var resource = RDF.GetResource(resourceID);
    var seqStats = new Object();

    seqStats.typeID = permissionTypes[index];
    seqStats.resourceID = resourceID;

    if (mzRDFCUtils.IsSeq(cbDatasource, resource))
      seqStats.entries = cbEntries(resourceID).GetCount();
    else {
      seqStats.entries = 0;
      failures++;
    }
    dsStatistics.push(seqStats);
  }
  return (failures == 7) ? null : dsStatistics;
}

function isFiltered(uri, type)
{
  var patterns = new Array();
  var url = uri.spec.match(/^www./i) ? uri.spec : "." + uri.spec;

  try {
    var pattern = pref.getComplexValue("multizilla.content-blocker.pattern.for." + type, Components.interfaces.nsISupportsString);
    patterns = pattern.data.replace(/\s/g, '').replace(/\./g, '\.+').replace(/\*/g, '.*');
    patterns = patterns.split(',');
    patterns = patterns.sort();
  } catch(ex) {
    return false;
  }
  if (patterns.length > 0) {
    for (var index = 0; index < patterns.length; index++) {
      var regexp = new RegExp();

      if (url.match(regexp.compile(patterns[index]),'i'))
        return true;
    }
  }
  return false;
}

function initWindowOpenType()
{
  try {
    gUseStackedWindows = pref.getBoolPref("multizilla.managers.use-stacked-windows");
  } catch(ex) {
    gUseStackedWindows = /Win/.test(navigator.platform);
  }
}

function onCancel()
{
  if (gEscapeKeyPressed) {
    // Prevent Permission Manager from closing when cookie data is displayed!
    if (gDisplayCookieData) {
      doBackButton();
      return false;
    }
  }
  // the default value for gConfirmForID is 0 and but should now be 2 or 4 (8???)
  return confirmClose(gConfirmForID); // return true or false (true will close the Permission Manager)
}

function confirmClose(aConfirm)
{
  var shouldClose = true;
  var confirmPrefID = "multizilla.perm-manager.confirm.quit";
  var dialogCheckboxText, confirmPrefValue = 15;
  /* ::::
     1  = Close Button
     2  = ESC key pressed
     4  = Ctrl-W
     8  = Click on [X] on Window
  :::: */
  try {
    // try to read the pref
    confirmPrefValue = pref.getIntPref(confirmPrefID);
  } catch(ex) {
    // Yikes, no pref found, add it with the default value
    pref.setIntPref(confirmPrefID, confirmPrefValue);
  }
  // Is the pref set for confirmation?
  if (confirmPrefValue && (confirmPrefValue & aConfirm) == aConfirm) {
    // Yes, ask the users confirmation
    switch(aConfirm) {
      case (aConfirm & 1): dialogCheckboxText = 'managerSkipConfirmForCloseButton';
              break;
      case (aConfirm & 2): dialogCheckboxText = 'managerSkipConfirmForEscapeKey';
              break;
      case (aConfirm & 4): dialogCheckboxText = 'managerSkipConfirmForCtrlW';
              break;
      case (aConfirm & 8): dialogCheckboxText = 'managerSkipConfirmForWindowX';
              break;
    }
    shouldClose = getConfirmation(window, 'pmConfirmCloseTitle', 'pmConfirmCloseText', dialogCheckboxText);
    // Is the checkbox checked and do we have a pref value to work with?
    if (gConfirmCheckbox && (confirmPrefValue >= aConfirm)) {
      // Yes, update pref
      pref.setIntPref(confirmPrefID, (confirmPrefValue -= aConfirm));
    }
  }
  return shouldClose;
}

function initTreeTooltip(aEvent)
{
  var node = document.tooltipNode; // <tree>, <treecols>, <treecol>, <splitter>, <treechildren>, <menupoup>, <menuitem>
  var nodeName = node.localName; // 'tree', 'treecols', 'treecol', 'splitter', 'treechildren', 'menupoup', 'menuitem'
  var tooltipNode = aEvent.target; // <tooltip>

  if (nodeName == 'tree' || nodeName.match('menu')) { // for <menupopup> and <menuitem> nodes
    aEvent.preventDefault();
    return;
  }
  tooltipNode.firstChild.hidden = (!node.hasAttribute('sort') || nodeName != 'treecol');
  tooltipNode.childNodes[1].hidden = (nodeName != 'treecols');
  tooltipNode.childNodes[2].hidden = (nodeName != 'splitter');
  tooltipNode.childNodes[3].hidden = (nodeName != 'treechildren');
  tooltipNode.lastChild.hidden = (nodeName != 'treechildren');
}

